#! /bin/sh

#set -x

#
# usage $0 mpeg-files+
#
if [ -z "$1" ] ; then
	printf "usage:\n"
	printf "\t $0 MPEG-FILE [MPEG-FILE]*\n"
	printf "\t\t where each file is one title, OR\n\n"
	printf "\t $0 -t Title1_a Title1_b -t Title2_a Title2_b\n"
	printf "\t\t where each title consist outta many files, OR\n\n"
	printf "\t $0 -T TITLENAME Title1_a Title1_b -T TITLENAME Title2_a Title2_b\n"
	printf "\t\t where each title consist outta many files with the given TITLENAME, OR\n\n"
	printf "\t $0 -T TITLENAME Title1_a Title1_b -t Title2_a Title2_b -X Title3 Title4\n"
	printf "\t\t mixed up, after -X each file is one title ...\n\n"
	exit 1
fi

#
# edit if necessary ..
#

DVDMENU_WORKINGDIR=/video/repackmpeg2
DVDMENU_RESULTDIR=/video/repackmpeg2/vobs
DVDMENU_TTF_FONTDIR=/usr/X11R6/lib/X11/fonts/truetype
DVDMENU_TTF_FONTFILE=arial.ttf

#
# nothing more to edit ... i believe ;-)
#

cd $DVDMENU_WORKINGDIR

declare -a titlesetnames
declare -a titlesetfiles
let titlesets=0

function make_titlename()
{
	title=$(basename $1 .vdr)
	title=$(basename $title .mpg)
	title=$(basename $title .vob)
	title=$(basename $title .mpeg)
	title=$(echo $title | sed -e 's/__.*//g' -e 's/_[0-9].*$//g' -e 's/_/ /g' -e 's/\./ /g')
	echo $title
}

#
#
#
function parse_titles()
{
	let titlesets=-1
	let titlesetfilesnum=-1

	files=$*

	while [ ! -z "$1" ] ; do
		i=$1
		shift

		if [ "$i" = "-X" ] ; then
			break;
		elif [ "$i" = "-T" ] ; then
			let titlesets=$titlesets+1
			let titlesetfilesnum=-1
		elif [ "$i" = "-t" ] ; then
			let titlesets=$titlesets+1
			let titlesetfilesnum=0
		elif [ $titlesets -ge 0 -a $titlesetfilesnum -eq -1 ] ; then
			titlesetnames[$titlesets]="$(make_titlename $i)"
			let titlesetfilesnum=$titlesetfilesnum+1
		elif [ $titlesets -ge 0 -a $titlesetfilesnum -ge 0 ] ; then
			if [ -z "${titlesetnames[$titlesets]}" ] ; then
				titlesetnames[$titlesets]="$(make_titlename $i)"
			fi
			titlesetfiles[$titlesets]="${titlesetfiles[$titlesets]} $i"
			let titlesetfilesnum=$titlesetfilesnum+1
		fi
	done

	let titlesets=$titlesets+1

	while [ ! -z "$1" ] ; do
		i=$1
		shift

		titlesetnames[$titlesets]="$(make_titlename $i)"
		titlesetfiles[$titlesets]="$i"

		let titlesets=$titlesets+1
	done

	if [ $titlesets -le 0 ] ; then
		for i in $files ; do
			titlesetnames[$titlesets]="$(make_titlename $i)"
			titlesetfiles[$titlesets]="$i"

			let titlesets=$titlesets+1
		done
	fi

	echo number of titlesets $titlesets
	
	let titleidx=0
	while [ $titleidx -lt $titlesets ] ; do
		echo "title set $titleidx: ${titlesetnames[$titleidx]}: ${titlesetfiles[$titleidx]}"
		let titleidx=$titleidx+1
	done

	if [ $titlesets -le 0 ] ; then
		echo ERROR no titleset given - $titlesets
		exit 1
	fi
}

function cleanup_dvdmenu()
{
	cd $DVDMENU_WORKINGDIR

	rm -f menu.m2v menu.mpg menu.jpg menu.sub menumask.png
	rm -f menu_audio*.mpg
	rm -f menu_audio*.m2v
	rm -f menu_audio*.jpg
	rm -f menu_audio*.sub
	rm -f menumask_audio*.png
}

function make_dvdmenu()
{
	cd $DVDMENU_WORKINGDIR

	rm -f dvdmenu.ini

	DVDMENU_WORKINGDIR_sed=$(echo $DVDMENU_WORKINGDIR | sed 's/\//\\\//g')
	DVDMENU_RESULTDIR_sed=$(echo $DVDMENU_RESULTDIR | sed 's/\//\\\//g')
	DVDMENU_TTF_FONTDIR_sed=$(echo $DVDMENU_TTF_FONTDIR | sed 's/\//\\\//g')
	DVDMENU_TTF_FONTFILE_sed=$(echo $DVDMENU_TTF_FONTFILE | sed 's/\//\\\//g')

	firstFileOfAll=$( echo "${titlesetfiles[0]}" | awk '{ printf $1 }')

	echo "detectin video_fps using 1st File Of All $firstFileOfAll"

	video_format="NTSC"
	let video_fps=$(tcprobe -i $firstFileOfAll  2>/dev/null | grep "frame rate" | awk  '{ printf $4 }' | sed 's/\..*$//g')

	echo "detected video_fps $video_fps"

	if [ $video_fps -eq 25 -o $video_fps -eq 50 ] ; then
		video_format="PAL"
	fi
	echo "assuming $video_format"

	cat dvdmenu-SKEL.ini | \
		sed -e "s/DVDMENU_WORKINGDIR/$DVDMENU_WORKINGDIR_sed/g" \
		    -e "s/DVDMENU_RESULTDIR/$DVDMENU_RESULTDIR_sed/g" \
		    -e "s/DVDMENU_TTF_FONTDIR/$DVDMENU_TTF_FONTDIR_sed/g" \
		    -e "s/DVDMENU_TTF_FONTFILE/$DVDMENU_TTF_FONTFILE_sed/g" \
		    -e "s/DVDMENU_FORMAT/$video_format/g" \
	    >> dvdmenu.ini 

	let titleidx=0
	while [ $titleidx -lt $titlesets ] ; do
		echo working titleset $titleidx: ${titlesetnames[$titleidx]}: ${titlesetfiles[$titleidx]}
		title=${titlesetnames[$titleidx]}

		firstFile=$( echo "${titlesetfiles[$titleidx]}" | awk '{ printf $1 }')

		audiotrack_number=$(tcprobe -i $firstFile 2>/dev/null | grep "audio track" | wc -l | awk '{ printf $1 }')

		audiotrack_types=$(tcprobe -i $firstFile 2>/dev/null | grep "audio track" | sed -e 's/^.*-n //g' -e 's/ .*//g' -e 's/0x2000/ac3/g' -e 's/0x50/mp2/g' | awk '{ printf "%s ",$1 }' )

		fnames="$(echo ${titlesetfiles[$titleidx]} | sed -e 's/\//\\\//g' -e 's/ /\\n/g')"

		let titleidx=$titleidx+1

		cat dvdmenu-title-SKEL.ini | \
			sed -e "s/SKEL_TITLE_IDX/$titleidx/g" \
			    -e "s/SKEL_TITLE_NAME/$title/g" \
			    -e "s/SKEL_TITLE_FILE/\<\<EOT\n$fnames\nEOT/g" \
			    -e "s/SKEL_TITLE_AUDIOTRACK_NUMBER/$audiotrack_number/g" \
		    >> dvdmenu.ini 

		let idx2=0
		for type in $audiotrack_types ; do
			echo "AudioTrackType$idx2=$type" >> dvdmenu.ini
			let idx2=$idx2+1
		done

	done

	isoname=$(echo "${titlesetnames[0]}" | sed -e 's/ /_/g')"."$(date +%Y-%m-%d-%H%M)".iso"
	echo resulting image name: $DVDMENU_RESULTDIR/$isoname

	./dvdmenu.pl dvdmenu.ini

	echo "rm -f $DVDMENU_RESULTDIR/$isoname" >> $DVDMENU_WORKINGDIR/dvdmenu_doit.sh

	echo "mv -v $DVDMENU_RESULTDIR/DVD_gen.iso $DVDMENU_RESULTDIR/$isoname" >> $DVDMENU_WORKINGDIR/dvdmenu_doit.sh

}

cleanup_dvdmenu

parse_titles $*

make_dvdmenu

nice -n 19 $DVDMENU_WORKINGDIR/dvdmenu_doit.sh

cleanup_dvdmenu


