#! /bin/sh

#set -x
#no_remove="no_remove"

THISDIR=$(pwd)

if [ -z "$1" -o -z "$2" -o -z "$3" ] ; then
	echo usage $0:
	echo 	out_basename requant_factor delete_input_files input-file1 input_file2
	echo OR
	echo 	out_basename requant_factor delete_input_files vdrcutter2 recording_directory mode
	exit 1
fi

bname_mpv_high=
bname_mpv_high_1=
bname_mpv_high_2=
bname_mpv_high_3=
bname_mpv_low_1=
bname_mpv_low_2=
bname_mpv_low_3=
bname_ac3_1=
bname_ac3_2=
bname_ac3_3=
bname_mp2_1=
bname_mp2_2=
bname_mp2_3=

tmpdir=
bname_out=
ifiles=

out_bname=$1
shift

bname_log="$out_bname".log

echo STARTING $0 - $out_bname - at $(date) > $bname_log 2>&1
echo STARTING $out_bname

mkdir -p vobs


requantfactor=$1
shift

delete_input_files=$1
shift


function repackmpeg2dplex() 
{
	echo
	echo de-plexing and concatenating PS streams to one set of es streams
	echo
	echo ./vdrsync.pl -o $tmpdir $ifiles
	time nice -n 19 ./vdrsync.pl -o $tmpdir $ifiles

	let i=0
	for esname in $tmpdir/*.mpv ; do
		let i=$i+1
		if [ $i -eq 1 ] ; then
			bname_mpv_high_1=$esname
			bname_mpv_low_1="$esname".low
		elif [ $i -eq 2 ] ; then
			bname_mpv_high_2=$esname
			bname_mpv_low_2="$esname".low
		elif [ $i -eq 3 ] ; then
			bname_mpv_high_3=$esname
			bname_mpv_low_3="$esname".low
		fi
	done

	let i=0
	for esname in $tmpdir/*.ac3 ; do
		let i=$i+1
		if [ $i -eq 1 ] ; then
			bname_ac3_1=$esname
		elif [ $i -eq 2 ] ; then
			bname_ac3_2=$esname
		elif [ $i -eq 3 ] ; then
			bname_ac3_3=$esname
		fi
	done

	let i=0
	for esname in $tmpdir/*.mpa ; do
		let i=$i+1
		if [ $i -eq 1 ] ; then
			bname_mp2_1=$esname
		elif [ $i -eq 2 ] ; then
			bname_mp2_2=$esname
		elif [ $i -eq 3 ] ; then
			bname_mp2_3=$esname
		fi
	done

	echo
	echo dplexed ES streams:
	echo
	test -e $bname_mpv_high_1 && echo $bname_mpv_high_1
	test -e $bname_mpv_high_2 && echo $bname_mpv_high_2
	test -e $bname_mpv_high_2 && echo $bname_mpv_high_2
	test -e $bname_ac3_1 && echo $bname_ac3_1
	test -e $bname_ac3_2 && echo $bname_ac3_2
	test -e $bname_ac3_3 && echo $bname_ac3_3
	test -e $bname_mp2_1 && echo $bname_mp2_1
	test -e $bname_mp2_2 && echo $bname_mp2_2
	test -e $bname_mp2_3 && echo $bname_mp2_3
	echo
	echo

        if [ ! -e $bname_mpv_high_1 ] ; then 
		echo dplex error ..
		echo resulting video ES not exist: $bname_mpv_high
		return 1
	fi
}

function repackmpeg2mplex() 
{
	echo
	echo repacking es streams to ps stream
	echo
	mplex_es=""

	if [ -e $bname_mpv_low_1 ] ; then
		mplex_es="$mplex_es $bname_mpv_low_1"
	elif [ -e $bname_mpv_high_1 ] ; then
		mplex_es="$mplex_es $bname_mpv_high_1"
	fi

	if [ -e $bname_mpv_low_2 ] ; then
		mplex_es="$mplex_es $bname_mpv_low_2"
	elif [ -e $bname_mpv_high_2 ] ; then
		mplex_es="$mplex_es $bname_mpv_high_2"
	fi

	if [ -e $bname_mpv_low_3 ] ; then
		mplex_es="$mplex_es $bname_mpv_low_3"
	elif [ -e $bname_mpv_high_3 ] ; then
		mplex_es="$mplex_es $bname_mpv_high_3"
	fi

	test -e $bname_ac3_1 && mplex_es="$mplex_es $bname_ac3_1"
	test -e $bname_ac3_2 && mplex_es="$mplex_es $bname_ac3_2"
	test -e $bname_ac3_3 && mplex_es="$mplex_es $bname_ac3_3"
	test -e $bname_mp2_1 && mplex_es="$mplex_es $bname_mp2_1"
	test -e $bname_mp2_2 && mplex_es="$mplex_es $bname_mp2_2"
	test -e $bname_mp2_3 && mplex_es="$mplex_es $bname_mp2_3"

	echo mplex -f 8 -S 0 -o $bname_out $mplex_es
	time nice -n 19 mplex -f 8 -S 0 -o $bname_out $mplex_es
}

function repackmpeg2requant() 
{
	factor=1.3

	if [ ! -z "$1" ] ; then
		factor=$1
	fi

	if [ -e $bname_mpv_high_1 ] ; then
		echo
		echo requant $factor in: $bname_mpv_high_1 out: $bname_mpv_low_1
		echo
		time nice -n 19 requant $factor < $bname_mpv_high_1 > $bname_mpv_low_1
	fi
	if [ -e $bname_mpv_high_2 ] ; then
		echo
		echo requant $factor in: $bname_mpv_high_2 out: $bname_mpv_low_2
		echo
		time nice -n 19 requant $factor < $bname_mpv_high_2 > $bname_mpv_low_2
	fi
	if [ -e $bname_mpv_high_3 ] ; then
		echo
		echo requant $factor in: $bname_mpv_high_3 out: $bname_mpv_low_3
		echo
		time nice -n 19 requant $factor < $bname_mpv_high_3 > $bname_mpv_low_3
	fi

	echo
	echo requant ES video streams:
	echo
	test -e $bname_mpv_low_1 && echo $bname_mpv_low_1
	test -e $bname_mpv_low_2 && echo $bname_mpv_low_2
	test -e $bname_mpv_low_3 && echo $bname_mpv_low_3

	echo
	echo
}

function repackmpeg_main() 
{
	echo $0 $requantfactor $ifiles >> $bname_log 2>&1

	if [ "$requantfactor" = "1" -o "$requantfactor" = "null" -o "$requantfactor" = "no" ] ; then
		echo INFO requant shrinking ONE - $requantfactor - no requanting >> $bname_log 2>&1
	else
		echo INFO requant shrinking about $requantfactor >> $bname_log 2>&1
	fi

	if [ -z "$no_remove" ] ; then rm -rfv $tmpdir >> $bname_log 2>&1 ; fi
	mkdir -p $tmpdir

	bname_mpv_high=$tmpdir/e0.mpv
	bname_mpv_high_1=
	bname_mpv_high_2=
	bname_mpv_high_3=
	bname_mpv_low_1=
	bname_mpv_low_2=
	bname_mpv_low_3=
	bname_ac3_1=
	bname_ac3_2=
	bname_ac3_3=
	bname_mp2_1=
	bname_mp2_2=
	bname_mp2_3=

	ok=0
	repackmpeg2dplex >> $bname_log 2>&1 && ok=1
	if [ $ok -eq 0 ] ; then 
		echo dplex failed exit - $out_bname
		if [ -z "$no_remove" ] ; then rm -rfv $tmpdir >> $bname_log 2>&1 ; fi
		return 1
	else
		echo dplex ok ..
	fi

	ok=0
	if [ "$requantfactor" = "1" -o "$requantfactor" = "null" -o "$requantfactor" = "no" ] ; then
		ok=1
		echo INFO no requanting - $requantfactor >> $bname_log 2>&1
	else
		repackmpeg2requant $requantfactor >> $bname_log 2>&1 && ok=1
	fi

	if [ $ok -eq 0 ] ; then 
		echo requant failed exit - $out_bname
		if [ -z "$no_remove" ] ; then rm -rfv $tmpdir >> $bname_log 2>&1 ; fi
		return 1
	else
		echo requant ok ..
	fi

	ok=0
	repackmpeg2mplex >> $bname_log 2>&1 && ok=1

	if [ $ok -eq 0 ] ; then 
		echo mplex failed exit - $out_bname
		if [ -z "$no_remove" ] ; then rm -rfv $tmpdir >> $bname_log 2>&1 ; fi
		return 1
	else
		echo mplex ok ..
	fi

	if [ -z "$no_remove" ] ; then rm -rfv $tmpdir >> $bname_log 2>&1 ; fi

	if [ "$delete_input_files" = "1" -o "$delete_input_files" = "true" -o \
             "$delete_input_files" = "yes" ] ; then
		echo deleting input files ...
		if [ -z "$no_remove" ] ; then rm -fv $ifiles >> $bname_log 2>&1 ; fi
	fi

	echo OK $0 - $out_bname - at $(date) >> $bname_log 2>&1
	echo OK $out_bname
	return 0
}


split=
icut_bname_temp=

if [ "$1" = "vdrcutter2" ] ; then
	shift

	rec_directory=$1
	shift

	mode=$1
	shift

	icut_bname_temp=vobs/"$out_bname"_temp
	if [ -z "$no_remove" ] ; then rm -rf $icut_bname_temp >> $bname_log 2>&1 ; fi
	mkdir -p $icut_bname_temp

	#
	# let vdrcutter cut the source first !
	#
        if [ "$mode" = "split" ] ; then
		# split files at marks
		vdrcutter2 -s -d $icut_bname_temp $rec_directory >> $bname_log 2>&1
		split="split"
	else
		# no split - just one file
		vdrcutter2 -d $icut_bname_temp $rec_directory >> $bname_log 2>&1
	fi

	ifiles=$icut_bname_temp/00*.vdr

else
	ifiles="$*"
fi

ok=0
if [ "$split" = "split" ] ; then

	ifiles_temp=$ifiles
	out_bname_save=$out_bname

	for ifiles in $ifiles_temp ; do
		out_bname="$out_bname_save"_$(basename $ifiles .vdr)
		bname_out=vobs/"$out_bname"_%d.mpg
		tmpdir=$THISDIR/"$out_bname"_temp
		ok=0
		repackmpeg_main && ok=1
		if [ $ok -eq 0 ] ; then  break; fi
	done
else
	bname_out=vobs/"$out_bname"_%d.mpg
	tmpdir=$THISDIR/"$out_bname"_temp
	ok=0
	repackmpeg_main && ok=1
fi

if [ $ok -eq 0 ] ; then 
	echo repackmpeg_main failed in $ifiles - exit - $out_bname
	echo repackmpeg_main failed in $ifiles - exit - $out_bname >> $bname_log 2>&1
	return 1
else
	if [ ! -z "$icut_bname_temp" ] ; then
		if [ -z "$no_remove" ] ; then rm -rfv $icut_bname_temp >> $bname_log 2>&1 ; fi
	fi
	echo repackmpeg_main OK
	echo repackmpeg_main OK >> $bname_log 2>&1
fi

