#! /bin/sh

#set -x
#no_remove="no_remove"

THISDIR=$(pwd)

if [ -z "$1" -o -z "$2" -o -z "$3" ] ; then
	echo usage $0:
	echo out_basename requant_factor delete_input_files input-file1 input_file2
	exit 1
fi

out_bname=$1
shift

mkdir -p vobs

bname_out=vobs/"$out_bname"_%d.mpg

requantfactor=$1
shift

delete_input_files=$1
shift

ifile=$1

bname=$(basename $ifile .mpg)
bname=$(basename $bname .vdr)
bname=$(basename $bname .vob)

bname_log="$out_bname".log

echo STARTING $0 - $out_bname - at $(date) > $bname_log 2>&1
echo STARTING $out_bname

echo $0 $requantfactor $ifile >> $bname_log 2>&1

if [ "$requantfactor" = "1" -o "$requantfactor" = "null" -o "$requantfactor" = "no" ] ; then
	echo INFO requant shrinking ONE - $requantfactor - no requanting >> $bname_log 2>&1
else
	echo INFO requant shrinking about $requantfactor >> $bname_log 2>&1
fi

tmpdir=$THISDIR/"$out_bname"_temp

if [ -z "$no_remove" ] ; then rm -rfv $tmpdir >> $bname_log 2>&1 ; fi
mkdir -p $tmpdir

video_file_high=$tmpdir/e0.mpv
video_file_low=$tmpdir/e0.low.mpv
audio_files=

function repackmpeg2dplex() 
{
	echo
	echo de-plexing and concatenating PS streams to one set of es streams
	echo

	audiotrack_number=$(tcprobe -i $ifile  2>/dev/null | grep "audio track" | wc -l | awk '{ printf $1 }')

	audiotrack_types=$(tcprobe -i $ifile 2>/dev/null | grep "audio track" | sed -e 's/^.*-n //g' -e 's/ .*//g' -e 's/0x2000/ac3/g' -e 's/0x50/mp2/g' | awk '{ printf "%s ",$1 }' )

	let idx2=0
	for type in $audiotrack_types ; do
		echo "extract audio $idx2 $type from $ifile to $tmpdir/audio$idx2.$type"
		tcextract -i $ifile -t vob -x $type -a $idx2 > $tmpdir/audio$idx2.$type
		audio_files="$audio_files $tmpdir/audio$idx2.$type"
		let idx2=$idx2+1
	done

	echo "extract video mpeg2 from $ifile to $video_file_high"
	tcextract -i $ifile -t vob -x mpeg2 >> $video_file_high

	if [ ! -e $video_file_high ] ; then 
		echo dplex error ..
		echo resulting video ES not exist: $video_file_high
		return 1
	fi
}

function repackmpeg2mplex() 
{
	echo
	echo repacking es streams to ps stream
	echo
	mplex_es=""

	if [ -e $video_file_low ] ; then
		mplex_es="$mplex_es $video_file_low"
	elif [ -e $video_file_high ] ; then
		mplex_es="$mplex_es $video_file_high"
	fi

	mplex_es="$mplex_es $audio_files"

	echo mplex -f 8 -S 0 -o $bname_out $mplex_es
	time nice -n 19 mplex -f 8 -S 0 -o $bname_out $mplex_es
}

function repackmpeg2requant() 
{
	factor=1.3

	if [ ! -z "$1" ] ; then
		factor=$1
	fi

	if [ -e $video_file_high ] ; then
		echo
		echo requant $factor in: $video_file_high out: $video_file_low
		echo
		time nice -n 19 requant $factor < $video_file_high > $video_file_low
	fi

	echo
	echo requant ES video streams:
	echo
	test -e $video_file_low && echo $video_file_low

	echo
	echo
}

ok=0
repackmpeg2dplex >> $bname_log 2>&1 && ok=1
if [ $ok -eq 0 ] ; then 
	echo dplex failed exit - $out_bname
	if [ -z "$no_remove" ] ; then rm -rfv $tmpdir >> $bname_log 2>&1 ; fi
	exit 1
else
	echo dplex ok ..
fi

ok=0
if [ "$requantfactor" = "1" -o "$requantfactor" = "null" -o "$requantfactor" = "no" ] ; then
	ok=1
	echo INFO no requanting - $requantfactor >> $bname_log 2>&1
else
	repackmpeg2requant $requantfactor >> $bname_log 2>&1 && ok=1
fi

if [ $ok -eq 0 ] ; then 
	echo requant failed exit - $out_bname
	if [ -z "$no_remove" ] ; then rm -rfv $tmpdir >> $bname_log 2>&1 ; fi
	exit 1
else
	echo requant ok ..
fi

ok=0
repackmpeg2mplex >> $bname_log 2>&1 && ok=1

if [ $ok -eq 0 ] ; then 
	echo mplex failed exit - $out_bname
	if [ -z "$no_remove" ] ; then rm -rfv $tmpdir >> $bname_log 2>&1 ; fi
	exit 1
else
	echo mplex ok ..
fi

if [ -z "$no_remove" ] ; then rm -rfv $tmpdir >> $bname_log 2>&1 ; fi

if [ "$delete_input_files" = "1" -o "$delete_input_files" = "true" -o "$delete_input_files" = "yes" ] ; then
	echo deleting input files ...
	if [ -z "$no_remove" ] ; then rm -fv $ifile >> $bname_log 2>&1 ; fi
fi

echo OK $0 - $out_bname - at $(date) >> $bname_log 2>&1
echo OK $out_bname

